<?php

require 'vendor/autoload.php';

// PHPExcel Bibliothek laden
use PhpOffice\PhpSpreadsheet\IOFactory;

// Router Bibliothek laden
use Bramus\Router\Router;

// Datenbankverbindung herstellen
$dsn = 'mysql:host=localhost;dbname=essensbestellung;charset=utf8';
$username = 'root';
$password = '';

try {
    $db = new PDO($dsn, $username, $password);
} catch (PDOException $e) {
    die('Verbindung zur Datenbank fehlgeschlagen: ' . $e->getMessage());
}

// Erstelle neuen Router
$router = new Router();

// Definiere eine Route
$router->get('/', function() {
    global $db;
    // Lösche die temporäre Excel-Datei
    if (file_exists('tmp/tabelle.xlsx')) {
        unlink('tmp/tabelle.xlsx');
    }

    // Lösche alle temporären PDF-Dateien
    $files = glob('tmp/pdfs/*.pdf');
    foreach ($files as $file) {
        if (is_file($file)){
            unlink($file);
        }
    }

    // Wenn tabelle.xlsx existiert, lösche diese
    if (file_exists('tmp/tabelle.xlsx')) {
        unlink('tmp/tabelle.xlsx');
    }

    // Überprüfe, ob die Tabelle "essensangebot" existiert
    $stmt = $db->prepare('SHOW TABLES LIKE "essensangebot"');
    $stmt->execute();
    $result = $stmt->fetch(PDO::FETCH_ASSOC);

    if ($result) {
        // Lösche die Tabelle "essensangebot"
        $stmt = $db->prepare('DROP TABLE essensangebot');
        $stmt->execute();
    }

    // Überprüfe, ob die Tabelle "essenseinwahl" existiert
    $stmt = $db->prepare('SHOW TABLES LIKE "essenseinwahl"');
    $stmt->execute();
    $result = $stmt->fetch(PDO::FETCH_ASSOC);

    if ($result) {
        // Lösche die Tabelle "essenseinwahl"
        $stmt = $db->prepare('DROP TABLE essenseinwahl');
        $stmt->execute();
    }

    require 'views/app.view.php';
});

// Mounte den Router auf /api
$router->mount('/api', function() use ($router) {
    // Definiere eine Route
    $router->post('/upload', function() {
        // Überprüfe, ob die hochgeladenen Dateien Excel-Dateien sind
        if ($_FILES['input_spreadsheet']['type'] != 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') {
            // Leite zur Startseite um
            echo "-1";
            exit;
        }

        // Überprüfe, ob die Datei bereits existiert
        if (file_exists('tmp/tabelle.xlsx')) {
            // Lösche die Datei
            unlink('tmp/tabelle.xlsx');
        }

        // Speichere die hochgeladene Datei in einem temporären Verzeichnis
        move_uploaded_file($_FILES['input_spreadsheet']['tmp_name'], 'tmp/tabelle.xlsx');

        // Pfad zur Excel-Datei
        $excelFile = 'tmp/tabelle.xlsx';

        // Excel-Datei öffnen
        $objPHPExcel = IOFactory::load($excelFile);

        // Arbeitsblatt auswählen
        $worksheet = $objPHPExcel->getSheetByName('Bestellungen');

        // Anzahl der Zeilen ermitteln aber leere Zeilen überspringen
        $highestRow = $worksheet->getHighestRow();
        $highestRow = $highestRow - 1;

        echo $highestRow;
        exit;
    });

    $router->post('/insert-angebot', function() {
        global $db;
        // Pfad zur Excel-Datei
        $excelFile = 'tmp/tabelle.xlsx';

        // Excel-Datei öffnen
        $objPHPExcel = IOFactory::load($excelFile);

        // Überprüfen, ob Tabelle bereits existiert
        $tableExists = $db->query('SHOW TABLES LIKE "essensangebot"')->rowCount() > 0;
        if ($tableExists) {
            // Tabelle bereits vorhanden, löschen
            $db->exec('DROP TABLE essensangebot');
        }

        // Tabelle in der MySQL-Datenbank erstellen
        $db->exec('CREATE TABLE essensangebot (
            id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
            tag VARCHAR(255),
            menu VARCHAR(255),
            preis VARCHAR(255)
        )');

        // Arbeitsblatt auswählen
        $worksheet = $objPHPExcel->getSheetByName('Essensangebot');

        // Zeilen durchgehen und Daten in MySQL-Datenbank speichern
        $i = 0;
        foreach ($worksheet->getRowIterator() as $row) {

            // Überschrift überspringen
            if ($i == 0){
                $i++;
                continue;
            }

            $cellIterator = $row->getCellIterator();
            $cellIterator->setIterateOnlyExistingCells(false); // alle Zellen durchgehen, auch leere

            $data = array();
            foreach ($cellIterator as $cell) {
                $data[] = $cell->getValue();
            }

            // Wenn die Zelle leer ist, dann aus dem Array entfernen
            $data = array_filter($data);

            // Wenn das Array leer ist, dann nächste Zeile
            if (empty($data)) {
                continue;
            }

            // Daten in MySQL-Datenbank speichern
            $stmt = $db->prepare('INSERT INTO essensangebot (tag, menu, preis) VALUES (?, ?, ?)');
            $stmt->execute($data);
        }

        // Verbindung zur MySQL-Datenbank schließen
        $db = null;

        echo "1";
    });

    $router->post('/insert-bestellungen', function() {
        global $db;

        // Pfad zur Excel-Datei
        $excelFile = 'tmp/tabelle.xlsx';

        // Überprüfen, ob Tabelle bereits existiert
        $tableExists = $db->query('SHOW TABLES LIKE "essenseinwahl"')->rowCount() > 0;
        if ($tableExists) {
            // Tabelle bereits vorhanden, löschen
            $db->exec('DROP TABLE essenseinwahl');
        }

        // Tabelle in der MySQL-Datenbank erstellen
        $db->exec('CREATE TABLE essenseinwahl (
            id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
            nachname VARCHAR(255),
            vorname VARCHAR(255),
            montag VARCHAR(255),
            dienstag VARCHAR(255),
            mittwoch VARCHAR(255),
            donnerstag VARCHAR(255),
            freitag VARCHAR(255)
        )');

        // Excel-Datei öffnen
        $objPHPExcel = IOFactory::load($excelFile);

        // Arbeitsblatt auswählen
        $worksheet = $objPHPExcel->getSheetByName('Bestellungen');

        // Zeilen durchgehen und Daten in MySQL-Datenbank speichern
        $i = 0;
        foreach ($worksheet->getRowIterator() as $row) {

            // Überschrift überspringen
            if ($i == 0){
                $i++;
                continue;
            }

            $cellIterator = $row->getCellIterator();
            $cellIterator->setIterateOnlyExistingCells(false); // alle Zellen durchgehen, auch leere

            $data = array();
            foreach ($cellIterator as $cell) {
                $data[] = $cell->getValue();
            }

            // Überprüfen, ob der Vor- oder Nachname leer ist
            if (empty($data[0]) || empty($data[1])) {
                continue;
            }

            // Bei den Essenswünschen die Leerzeichen entfernen
            $data[2] = str_replace(' ', '', $data[2]);
            $data[3] = str_replace(' ', '', $data[3]);
            $data[4] = str_replace(' ', '', $data[4]);
            $data[5] = str_replace(' ', '', $data[5]);
            $data[6] = str_replace(' ', '', $data[6]);

            // Leere Werte durch none ersetzen
            if (empty($data[2])) {
                $data[2] = 'none';
            }
            if (empty($data[3])) {
                $data[3] = 'none';
            }
            if (empty($data[4])) {
                $data[4] = 'none';
            }
            if (empty($data[5])) {
                $data[5] = 'none';
            }
            if (empty($data[6])) {
                $data[6] = 'none';
            }

            // Daten in MySQL-Datenbank speichern
            $stmt = $db->prepare('INSERT INTO essenseinwahl (nachname, vorname, montag, dienstag, mittwoch, donnerstag, freitag) VALUES (?, ?, ?, ?, ?, ?, ?)');
            $stmt->execute($data);
        }

        // Verbindung zur MySQL-Datenbank schließen
        $db = null;

        echo "1";
    });

    $router->post('/generate-letter', function () {
        global $db;

        // Deaktiviere die Fehlermeldung
        error_reporting(0);

        $mpdf = new \Mpdf\Mpdf([
            'default_font' => 'Arial',
            'tempDir' => __DIR__ . '/tmp'
        ]);

        // Lade alle Bestellungen aus der Datenbank
        $stmt = $db->prepare('SELECT * FROM essenseinwahl');
        $stmt->execute();
        $bestellungen = $stmt->fetchAll();

        foreach ($bestellungen as $bestellung){
            $brutto = 0;
            $date = date('d.m.Y');

            $letter = '
            <style>
                table {
                    border-collapse: collapse;
                    width: 100%;
                }
                td, th {
                    border: 1px solid black;
                    padding: 5px;
                }
            </style>
            <h1 style="margin-bottom: 0px;">Rechnung (# ' . $bestellung['id'] . ')</h1>
            <i class="display: block;">Rechnungsdatum: ' . $date . '</i>
            <br><br><br>
            <p>
                Hallo ' . htmlspecialchars($bestellung['vorname']) . ' ' . htmlspecialchars($bestellung['nachname']) . ',<br>
                vielen Dank für deine Bestellung. Hier findest du deine Rechnung. <br>
                Bitte überweise den Betrag auf folgendes Konto:<br>
                <br>
                <b>Empfänger:</b> Max Mustermann<br>
                <b>IBAN:</b> DE12345678901234567890<br>
                <b>BIC:</b> ABCDDEFF<br>
                <b>Verwendungszweck:</b> Rechnung # ' . $bestellung['id'] . '<br>
            </p>
            <table>
                <tr>
                    <td style="font-weight: bold;">Tag</td>
                    <td style="font-weight: bold;">Menu</td>
                    <td style="font-weight: bold;">Preis</td>
                </tr>
            ';

            // Bestellung für Montag
            if ($bestellung['montag'] != 'none') {
                $stmt = $db->prepare('SELECT * FROM essensangebot WHERE tag = ?');
                $stmt->execute(array($bestellung['montag']));
                $angebot = $stmt->fetch();

                $letter = $letter . '
                    <tr>
                        <td>Montag</td>
                        <td>' . $angebot['menu'] . '</td>
                        <td>' . $angebot['preis'] . ' €</td>
                    </tr>
                ';
                
                $brutto = $brutto + $angebot['preis'];
            }

            // Bestellung für Dienstag
            if ($bestellung['dienstag'] != 'none') {
                $stmt = $db->prepare('SELECT * FROM essensangebot WHERE tag = ?');
                $stmt->execute(array($bestellung['dienstag']));
                $angebot = $stmt->fetch();

                $letter = $letter . '
                    <tr>
                        <td>Dienstag</td>
                        <td>' . $angebot['menu'] . '</td>
                        <td>' . $angebot['preis'] . ' €</td>
                    </tr>
                ';
                
                $brutto = $brutto + $angebot['preis'];
            }

            // Bestellung für Mittwoch
            if ($bestellung['mittwoch'] != 'none') {
                $stmt = $db->prepare('SELECT * FROM essensangebot WHERE tag = ?');
                $stmt->execute(array($bestellung['mittwoch']));
                $angebot = $stmt->fetch();

                $letter = $letter . '
                    <tr>
                        <td>Mittwoch</td>
                        <td>' . $angebot['menu'] . '</td>
                        <td>' . $angebot['preis'] . ' €</td>
                    </tr>
                ';
                
                $brutto = $brutto + $angebot['preis'];
            }

            // Bestellung für Donnerstag
            if ($bestellung['donnerstag'] != 'none') {
                $stmt = $db->prepare('SELECT * FROM essensangebot WHERE tag = ?');
                $stmt->execute(array($bestellung['donnerstag']));
                $angebot = $stmt->fetch();

                $letter = $letter . '
                    <tr>
                        <td>Donnerstag</td>
                        <td>' . $angebot['menu'] . '</td>
                        <td>' . $angebot['preis'] . ' €</td>
                    </tr>
                ';
                
                $brutto = $brutto + $angebot['preis'];
            }

            // Bestellung für Freitag
            if ($bestellung['freitag'] != 'none') {
                $stmt = $db->prepare('SELECT * FROM essensangebot WHERE tag = ?');
                $stmt->execute(array($bestellung['freitag']));
                $angebot = $stmt->fetch();

                $letter = $letter . '
                    <tr>
                        <td>Freitag</td>
                        <td>' . $angebot['menu'] . '</td>
                        <td>' . $angebot['preis'] . ' €</td>
                    </tr>
                ';
                
                $brutto = $brutto + $angebot['preis'];
            }

            $netto = $brutto / 1.19;
            $netto = round($netto, 2);
            $mwst = $brutto - $netto;
            $mwst = round($mwst, 2);
            $brutto = round($brutto, 2);
            
            $letter = $letter .
            '
                    </table>
                    <br>
                    <table>
                        <tr>
                            <td>Netto</td>
                            <td>' . $netto . ' €</td>
                        </tr>
                        <tr>
                            <td>Mwst (19 %)</td>
                            <td>' . $mwst . ' €</td>
                        </tr>
                        <tr>
                            <td>Fälliger Betrag</td>
                            <td>' . $brutto . ' €</td>
                        </tr>
                    </table>
                    <br><br>
                    <p>
                        Mit freundlichen Grüßen<br>
                        <br>
                        <b>Max Mustermann</b><br>
                        <i>Essenslieferant</i>
                    </p>
                ';
            $mpdf->WriteHTML($letter);
            $mpdf->AddPage();
        }

        // Lösche alle Dateien im Ordner tmp/pdfs
        $files = glob('tmp/pdfs/*');

        // Durchlaufe alle Dateien
        foreach ($files as $file){
            unlink($file);
        }


        $filename = bin2hex(random_bytes(32));
        $filename = 'bestellungen-' . $filename . '.pdf';
        $mpdf->Output('tmp/pdfs/' . $filename, \Mpdf\Output\Destination::FILE);

        echo $filename;
    });

    $router->post('/remove-artifacts', function () {
        global $db;

        // Lösche tmp/tabelle.xlsx
        unlink('tmp/tabelle.xlsx');

        // Drop essensangebot
        $stmt = $db->prepare('DROP TABLE essensangebot');
        $stmt->execute();

        // Drop essenseinwahl
        $stmt = $db->prepare('DROP TABLE essenseinwahl');
        $stmt->execute();

        echo "1";
    });

    $router->get('/download-letter', function () {
        $filename = $_GET['filename'];
        
        // Entferne .pdf von $filename
        $filename = str_replace('.pdf', '', $filename);

        // Entferne alle Sonderzeichen
        $filename = preg_replace('/[^A-Za-z0-9\-]/', '', $filename);

        // Füge .pdf wieder hinzu
        $filename = $filename . '.pdf';

        // Definiere den Pfad zur Datei
        $file = 'tmp/pdfs/' . $filename;

        // Prüfe ob die Datei existiert
        if (file_exists($file)) {
            // Lade die Datei herunter
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($file) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));
            readfile($file);
            unlink($file);
            exit;
        }
    });

});

// Definiere eine 404 Route
$router->set404(function() {
    // Leite zur Startseite um
    header('Location: /');
});

// Führe den Router aus
$router->run();